/* Error # */

#include	<stdio.h>

//---"AddFrame" return value---
#define AF_DONE			0
#define AF_ER_NOSPACE	1
#define AF_ER_NUMRANGE	2

//---"Go" return value---
#define GO_DONE			0
#define GO_ER_NOFRM		1
#define GO_ER_TXTOUT	2
#define GO_ER_ROOT		3
#define GO_ER_RXTOUT	4
#define GO_ER_RXOVER	5

//---"ReadFrame" return value---
#define RF_DONE			0
#define RF_CAMAC		1
#define RF_DAQCTRL		2
#define RF_INT_LAM		3
#define RF_INT_TRG		4
#define RF_WAIT_TRG		5
#define RF_ER_RANGEOUT	6
#define RF_ER_NODATA	7

//---"Wait Intrupt Frame" return value---
#define WI_DONE			0
#define WI_ER_TOUT		1
#define WI_ER_RXOVER	2
#define WI_ER_NOMALFRM	3
#define WI_ER_NOMATCH	4



//------others Error Codes-------
#define NO_ERR		0
#define ERR_PRIV	1
#define ERR_EMP		2
#define ERR_FNAME	3
#define ERR_TXTOUT	4
#define ERR_CMP		5
#define ERR_RXTOUT	6


